/*
 * 文 件 名:  CompareUtils.java
 * 版    权:  浩鲸云科技股份有限公司
 * 描    述:  <描述>
 * 修 改 人:  LvWenbin
 * 修改时间:  2018/11/19 16:47
 */
package com.huashe.citybrain.commons.base.utils;

import java.util.Queue;
import java.util.concurrent.ArrayBlockingQueue;

/**
 * $比较工具
 *
 * @author LvWenbin
 * @version [版本号, 2018/11/19 16:47]
 * @see [相关类/方法]
 * @since [产品/模块版本]
 */
public abstract class CompareUtils {
    /**
     * $封装待比较参数 <功能详细描述>
     *
     * @return
     * @see [类、类#方法、类#成员]
     */
    public static Queue<Object> buildQueue(Object... objs) {
        Queue<Object> queue = new ArrayBlockingQueue<Object>(objs.length);

        for (Object tmpObj : objs) {
            queue.offer(tmpObj);
        }

        return queue;
    }

    /**
     * $多条件递归比较 <功能详细描述>
     *
     * @param q1
     * @param q2
     * @return
     * @see [类、类#方法、类#成员]
     */
    public static int compare(Queue<Object> q1, Queue<Object> q2) {
        int cValue = 0;
        Object obj1 = q1.poll();
        Object obj2 = q2.poll();

        if (obj1 != null && obj2 != null) {
            cValue = getCValue(obj1, obj2);

            if (cValue == 0) {
                cValue = compare(q1, q2);
            }
        }

        return cValue;
    }

    private static int getCValue(Object obj1, Object obj2) {
        int cValue = 0;
        if (obj1 instanceof Integer && obj2 instanceof Integer) {
            cValue = intCompare((Integer) obj1, (Integer) obj2);
        }
        else if (obj1 instanceof String && obj2 instanceof String) {
            cValue = stringCompare((String) obj1, (String) obj2);
        }
        else if (obj1 instanceof Long && obj2 instanceof Long) {
            cValue = longCompare((Long) obj1, (Long) obj2);
        }
        else if (obj1 instanceof Double && obj2 instanceof Double) {
            cValue = doubleCompare((Double) obj1, (Double) obj2);
        }

        return cValue;
    }

    private static int intCompare(Integer value1, Integer value2) {
        return value1.compareTo(value2);
    }

    private static int stringCompare(String value1, String value2) {
        return value1.compareTo(value2);
    }

    private static int longCompare(Long value1, Long value2) {
        return value1.compareTo(value2);
    }

    private static int doubleCompare(Double value1, Double value2) {
        return value1.compareTo(value2);
    }
}
