/*
 * 文 件 名:  DataSourceUtils
 * 版    权:  浩鲸云计算科技股份有限公司
 * 描    述:  <描述>
 * 修 改 人:  learshaw
 * 修改时间:  2019/7/16
 * 跟踪单号:  <跟踪单号>
 * 修改单号:  <修改单号>
 * 修改内容:  <修改内容>
 */
package com.huashe.citybrain.commons.base.utils;

import com.alibaba.druid.pool.DruidDataSource;
import org.apache.ibatis.session.SqlSessionFactory;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;

import javax.sql.DataSource;

/**
 * DataSource工具类 <功能详细描述>
 *
 * @author learshaw
 * @version [版本号, 2019/7/16]
 * @see [相关类/方法]
 * @since [产品/模块版本]
 */
public abstract class DataSourceUtils {
    private static final Logger log = LoggerFactory.getLogger(DataSourceUtils.class);

    /**
     * 数据源配置
     *
     * @param driverClass 驱动
     * @param url         地址
     * @param user        用户名
     * @param password    密码
     * @return 数据源
     */
    public static DruidDataSource buildDataSource(String driverClass, String url, String user, String password) {
        DruidDataSource dataSource = new DruidDataSource();
        dataSource.setDriverClassName(driverClass);
        dataSource.setUrl(url);
        dataSource.setUsername(user);
        dataSource.setPassword(password);

        return dataSource;
    }

    public static SqlSessionFactory buildSqlSessionFactory(DataSource dataSource, String mapperLocation,
        String mybatisConfig) {
        SqlSessionFactory factory = null;
        try {
            SqlSessionFactoryBean sessionFactory = new SqlSessionFactoryBean();
            sessionFactory.setDataSource(dataSource);
            sessionFactory.setMapperLocations(new PathMatchingResourcePatternResolver().getResources(mapperLocation));
            sessionFactory.setConfigLocation(new PathMatchingResourcePatternResolver().getResource(mybatisConfig));
            factory = sessionFactory.getObject();
        }
        catch (Exception e) {
            log.error("buildSqlSessionFactory fail!", e);
        }

        return factory;
    }
}