/*
 * 文 件 名:  XlsxExcelUtils
 * 版    权:  浩鲸云计算科技股份有限公司
 * 描    述:  <描述>
 * 修 改 人:  learshaw
 * 修改时间:  2020/3/17
 * 跟踪单号:  <跟踪单号>
 * 修改单号:  <修改单号>
 * 修改内容:  <修改内容>
 */
package com.huashe.citybrain.commons.base.utils;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.CellValue;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFCellStyle;
import org.apache.poi.xssf.usermodel.XSSFFormulaEvaluator;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;

/**
 * XSSFWorkbook主要用于解析xlsx
 * 依赖poi-ooxml
 * <功能详细描述>
 *
 * @author learshaw
 * @version [版本号, 2020/3/17]
 * @see [相关类/方法]
 * @since [产品/模块版本]
 */
public abstract class ExcelXlsxUtils {
    /**
     * 日志记录
     */
    private static final Log logger = LogFactory.getLog(ExcelXlsUtils.class);

    /**
     * 读取excel <功能详细描述>
     *
     * @param filePath 文件路径
     * @return
     * @see [类、类#方法、类#成员]
     */
    public static XSSFWorkbook readXls(String filePath) {
        InputStream inputStream = null;
        XSSFWorkbook workBook = null;

        try {
            inputStream = new FileInputStream(filePath);
            workBook = new XSSFWorkbook(inputStream);
        }
        catch (FileNotFoundException e) {
            logger.error("readXlsx fail!", e);
        }
        catch (Exception e) {
            logger.error("readXlsx fail!", e);
        }
        finally {
            CloseUtils.close(inputStream);
        }

        return workBook;
    }

    /**
     * 写Excel <功能详细描述>
     *
     * @param xssfWorkbook excel文档对象
     * @param path         存放路径
     * @see [类、类#方法、类#成员]
     */
    public static void writeXls(XSSFWorkbook xssfWorkbook, String path) {
        ExcelUtils.writeXls(xssfWorkbook, path);
    }

    /**
     * 获取输入流
     * <功能详细描述>
     *
     * @param xssfWorkBook xssfWorkBook
     * @return
     * @see [类、类#方法、类#成员]
     */
    public static InputStream getInputStream(XSSFWorkbook xssfWorkBook) {
        return ExcelUtils.getInputStream(xssfWorkBook);
    }

    /**
     * 获取字节数组输出流
     * <功能详细描述>
     *
     * @param xssfWorkBook
     * @return
     * @see [类、类#方法、类#成员]
     */
    public static ByteArrayOutputStream getByteArrayOutputStream(XSSFWorkbook xssfWorkBook) {
        return ExcelUtils.getByteArrayOutputStream(xssfWorkBook);
    }

    /**
     * 获取sheet <功能详细描述>
     *
     * @param workBook  Excel工作薄
     * @param sheetName Sheet名称
     * @return
     * @see [类、类#方法、类#成员]
     */
    public static XSSFSheet getSheet(XSSFWorkbook workBook, String sheetName) {
        XSSFSheet sheet = workBook.getSheet(sheetName);

        if (null == sheet) {
            sheet = workBook.createSheet(sheetName);
        }

        return sheet;
    }

    /**
     * 获取行
     * <功能详细描述>
     *
     * @param rowIndex 行号
     * @return
     * @see [类、类#方法、类#成员]
     */
    public static XSSFRow getRow(XSSFSheet sheet, int rowIndex) {
        XSSFRow row = sheet.getRow(rowIndex);

        if (row == null) {
            row = sheet.createRow(rowIndex);
        }

        return row;
    }

    /**
     * 获取单元格
     * <功能详细描述>
     *
     * @param row      行对象
     * @param colIndex 列编号
     * @return
     * @see [类、类#方法、类#成员]
     */
    public static XSSFCell getCell(XSSFRow row, int colIndex) {
        XSSFCell cell = row.getCell(colIndex);

        if (cell == null) {
            cell = row.createCell(colIndex);
        }

        return cell;
    }

    /**
     * 获取单元格
     * <功能详细描述>
     *
     * @param sheet    sheet页对象
     * @param rowIndex 行编号
     * @param colIndex 列编号
     * @return
     * @see [类、类#方法、类#成员]
     */
    public static XSSFCell getCell(XSSFSheet sheet, int rowIndex, int colIndex) {
        XSSFRow row = getRow(sheet, rowIndex);
        return getCell(row, colIndex);
    }

    /**
     * 填充单元格值 <功能详细描述>
     *
     * @param sheet    Sheet名称
     * @param rowIndex 行坐标
     * @param colIndex 列坐标
     * @param value    填充值
     * @param cellType 单元格参数类型
     * @see [类、类#方法、类#成员]
     */
    public static void setSheetCellValue(XSSFSheet sheet, int rowIndex, int colIndex, String value, CellType cellType) {
        XSSFCell cell = getCell(sheet, rowIndex, colIndex);
        setCellValue(cell, value, cellType);
    }

    /**
     * 填充单元格值 <功能详细描述>
     *
     * @param sheet    Sheet名称
     * @param rowIndex 行坐标
     * @param colIndex 列坐标
     * @param value    填充值
     * @see [类、类#方法、类#成员]
     */
    public static void setSheetCellValue(XSSFSheet sheet, int rowIndex, int colIndex, int value) {
        XSSFCell cell = getCell(sheet, rowIndex, colIndex);
        setCellValue(cell, value);
    }

    /**
     * 填充单元格值 <功能详细描述>
     *
     * @param sheet    Sheet名称
     * @param rowIndex 行坐标
     * @param colIndex 列坐标
     * @param value    填充值
     * @see [类、类#方法、类#成员]
     */
    public static void setSheetCellValue(XSSFSheet sheet, int rowIndex, int colIndex, long value) {
        XSSFCell cell = getCell(sheet, rowIndex, colIndex);
        setCellValue(cell, value);
    }

    /**
     * 填充单元格值 <功能详细描述>
     *
     * @param sheet    Sheet名称
     * @param rowIndex 行坐标
     * @param colIndex 列坐标
     * @param value    填充值
     * @see [类、类#方法、类#成员]
     */
    public static void setSheetCellValue(XSSFSheet sheet, int rowIndex, int colIndex, String value) {
        XSSFCell cell = getCell(sheet, rowIndex, colIndex);
        setCellValue(cell, value);
    }

    /**
     * 填充单元格值 <功能详细描述>
     *
     * @param xssfCell 单元格
     * @param value    填充值
     * @param cellType 单元格类型
     * @see [类、类#方法、类#成员]
     */
    public static void setCellValue(XSSFCell xssfCell, String value, CellType cellType) {
        ExcelUtils.setCellValue(xssfCell, value, cellType);
    }

    /**
     * 填充单元格值 <功能详细描述>
     *
     * @param cell  单元格
     * @param value 填充值
     * @see [类、类#方法、类#成员]
     */
    public static void setCellValue(XSSFCell cell, int value) {
        cell.setCellType(CellType.NUMERIC);
        cell.setCellValue(value);
    }

    /**
     * 填充单元格值 <功能详细描述>
     *
     * @param cell  单元格
     * @param value 填充值
     * @see [类、类#方法、类#成员]
     */
    public static void setCellValue(XSSFCell cell, long value) {
        cell.setCellType(CellType.NUMERIC);
        cell.setCellValue(value);
    }

    /**
     * 填充单元格值 <功能详细描述>
     *
     * @param cell  单元格
     * @param value 填充值
     * @see [类、类#方法、类#成员]
     */
    public static void setCellValue(XSSFCell cell, String value) {
        cell.setCellType(CellType.STRING);
        cell.setCellValue(value);
    }

    public static String getSheetCellValue(XSSFSheet sheet, int rowIndex, int colIndex) {
        XSSFCell cell = getCell(sheet, rowIndex, colIndex);
        return getCellStringValue(cell);
    }

    /**
     * 读取单元格值
     * <功能详细描述>
     *
     * @param xssfCell 单元格
     * @return String
     * @see [类、类#方法、类#成员]
     */
    public static String getCellStringValue(XSSFCell xssfCell) {
        return ExcelUtils.getCellStringValue(xssfCell);
    }

    /**
     * 合并单元格
     * <功能详细描述>
     *
     * @param sheet      sheet页
     * @param rowFrom    开始行
     * @param rowTo      结束行
     * @param columnFrom 开始列
     * @param columnTo   结束列
     * @see [类、类#方法、类#成员]
     */
    public static void mergeCell(XSSFSheet sheet, int rowFrom, int rowTo, int columnFrom, int columnTo) {
        CellRangeAddress crAddress = new CellRangeAddress(rowFrom, rowTo, columnFrom, columnTo);
        sheet.addMergedRegion(crAddress);
    }

    /**
     * 设置单元格格式
     * <功能详细描述>
     *
     * @see [类、类#方法、类#成员]
     */
    public static void setCellStyle(XSSFWorkbook workBook, XSSFCell cell, HorizontalAlignment styleType) {
        XSSFCellStyle cellStyle = workBook.createCellStyle();
        cellStyle.setAlignment(styleType);
        cell.setCellStyle(cellStyle);
    }

    /**
     * 计算出单元格中公式的值
     * <功能详细描述>
     *
     * @param workBook 工作部
     * @param cell     单元格
     * @see [类、类#方法、类#成员]
     */
    public static double calculateFormulaValue(XSSFWorkbook workBook, XSSFCell cell) {
        double retValue = 0.0;

        if (cell.getCellType() == CellType.FORMULA) {
            XSSFFormulaEvaluator evaluator = new XSSFFormulaEvaluator(workBook);
            CellValue cellValue = evaluator.evaluate(cell);
            retValue = cellValue.getNumberValue();
        }

        return retValue;
    }

    /**
     * 重设公式单元格值
     * <功能详细描述>
     *
     * @param sheet    sheet
     * @param rowIndex rowIndex
     * @Param colIndex colIndex
     * @see [类、类#方法、类#成员]
     */
    public static void resetFormulaCell(XSSFSheet sheet, int rowIndex, int colIndex) {
        XSSFCell hssfCell = getCell(sheet, rowIndex, colIndex);

        if (CellType.FORMULA == hssfCell.getCellType()) {
            //取得公式单元格的公式,重新设置
            hssfCell.setCellFormula(hssfCell.getCellFormula());
        }
    }

    /**
     * 重设公式单元格值
     * <功能详细描述>
     *
     * @param hssfCell hssfCell
     * @see [类、类#方法、类#成员]
     */
    public static void resetFormulaCell(XSSFCell hssfCell) {
        if (CellType.FORMULA == hssfCell.getCellType()) {
            //取得公式单元格的公式,重新设置
            hssfCell.setCellFormula(hssfCell.getCellFormula());
        }
    }
}