/*
 * 文 件 名:  DistanceUtils.java
 * 版    权:  浩鲸云科技股份有限公司
 * 描    述:  <描述>
 * 修 改 人:  LvWenbin
 * 修改时间:  2018/11/06 18:51
 */
package com.huashe.citybrain.commons.base.utils;

import org.springframework.data.geo.Point;

import java.util.List;

/**
 * $距离工具 <功能详细描述>
 *
 * @author LvWenbin
 * @version [版本号, 2018/11/06 18:51]
 * @see [相关类/方法]
 * @since [产品/模块版本]
 */
public abstract class GeoUtils {
    /**
     * $地球半径
     */
    private static final double EARTH_RADIUS = 6378.137;

    /**
     * $取两点之间距离 coordinate1 坐标1 coordinate2 坐标2
     *
     * @return 两个经纬度的距离
     */
    public static double getDistance(Point coordinate1, Point coordinate2) {
        double firstRadLng = rad(coordinate1.getX());
        double firstRadLat = rad(coordinate1.getY());
        double secondRadLng = rad(coordinate2.getX());
        double secondRadLat = rad(coordinate2.getY());

        double a = firstRadLat - secondRadLat;
        double b = firstRadLng - secondRadLng;
        double cal = 2 * Math.asin(Math.sqrt(
            Math.pow(Math.sin(a / 2), 2) + Math.cos(firstRadLat) * Math.cos(secondRadLat) * Math
                .pow(Math.sin(b / 2), 2))) * EARTH_RADIUS;
        double result = Math.round(cal * 10000d) / 10000d;
        return result * 1000;
    }

    private static double rad(double d) {
        return d * Math.PI / 180.0;
    }

    /**
     * $根据输入的地点坐标计算中心点 <适用于400KM以上场景>
     *
     * @param geoCoordinateList
     * @return
     */
    public static Point getCenterPoint(List<Point> geoCoordinateList) {
        int total = geoCoordinateList.size();
        double X = 0;
        double Y = 0;
        double Z = 0;

        for (Point g : geoCoordinateList) {
            double lat;
            double lon;
            double x;
            double y;
            double z;
            lon = g.getX() * Math.PI / 180;
            lat = g.getY() * Math.PI / 180;
            x = Math.cos(lat) * Math.cos(lon);
            y = Math.cos(lat) * Math.sin(lon);
            z = Math.sin(lat);
            X += x;
            Y += y;
            Z += z;
        }

        X = X / total;
        Y = Y / total;
        Z = Z / total;
        double Lon = Math.atan2(Y, X);
        double Hyp = Math.sqrt(X * X + Y * Y);
        double Lat = Math.atan2(Z, Hyp);
        return new Point(Lon * 180 / Math.PI, Lat * 180 / Math.PI);
    }

    /**
     * $根据输入的地点坐标计算中心点 <适用于400KM以下场景>
     *
     * @param geoCoordinateList
     * @return
     */
    public static Point getCenterPoint400(List<Point> geoCoordinateList) {
        // 以下为简化方法（400km以内）
        int total = geoCoordinateList.size();
        double lat = 0;
        double lon = 0;

        for (Point g : geoCoordinateList) {
            lon += g.getX() * Math.PI / 180;
            lat += g.getY() * Math.PI / 180;

        }
        lat /= total;
        lon /= total;
        return new Point(lon * 180 / Math.PI, lat * 180 / Math.PI);
    }
}
