/*
 * 文 件 名:  Sha256Utils
 * 版    权:  浩鲸云计算科技股份有限公司
 * 描    述:  <描述>
 * 修 改 人:  0027008907/lv.wenbin
 * 修改时间:  2020/10/28
 * 跟踪单号:  <跟踪单号>
 * 修改单号:  <修改单号>
 * 修改内容:  <修改内容>
 */
package com.huashe.citybrain.commons.base.utils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 * Sha256Utils
 * <功能详细描述>
 *
 * @author 0027008907/lv.wenbin
 * @version [版本号, 2020/10/28]
 * @see [相关类/方法]
 * @since [产品/模块版本]
 */
public abstract class Sha256Utils {
    /**
     * 日志
     */
    public static final Logger log = LoggerFactory.getLogger(Sha256Utils.class);

    /**
     * 利用java原生的类实现SHA256加密
     *
     * @param str 加密后的报文
     * @return
     */
    public static String getSHA256(String str) {
        MessageDigest messageDigest;
        String encodestr = "";

        try {
            messageDigest = MessageDigest.getInstance("SHA-256");
            messageDigest.update(str.getBytes(StandardCharsets.UTF_8));
            encodestr = byte2Hex(messageDigest.digest());
        }
        catch (NoSuchAlgorithmException e) {
            log.error("getSHA256 fail!", e);
        }

        return encodestr;
    }

    /**
     * 将byte转为16进制
     *
     * @param bytes 字节
     * @return 结果
     */
    private static String byte2Hex(byte[] bytes) {
        StringBuilder stringBuilder = new StringBuilder();
        String temp = null;

        for (int i = 0; i < bytes.length; i++) {
            temp = Integer.toHexString(bytes[i] & 0xFF);

            if (temp.length() == 1) {
                // 1得到一位的进行补0操作
                stringBuilder.append("0");
            }

            stringBuilder.append(temp);
        }

        return stringBuilder.toString();
    }
}